@@ -27,8 +27,6 @@ cmps <- list(
   fs[[1]] %>>>% !!(fs[[2]] %>>>% fs[[3]])
 )
 
-context("Composing functions")
-
 test_that("empty or NULL composition yields NULL", {
   expect_null(compose())
   expect_null(compose(NULL))
@@ -126,8 +124,8 @@ test_that("compositions are called as flattened pipeline", {
   # Body reveals flattened pipeline
   expect_length(unlist(as.list(baz)), 6)
   expect_identical(
-    body(baz),
-    quote(`__6__`(`__5__`(`__4__`(`__3__`(`__2__`(`__1__`(..., . = .)))))))
+    body(environment(baz)$`__cmp__`),
+    quote(`__6__`(`__5__`(`__4__`(`__3__`(`__2__`(`__1__`(...)))))))
   )
 })
 
@@ -436,8 +434,6 @@ test_that("point is matched when a symbol not a character (#27)", {
   expect_identical(f(letters), paste(letters, collapse = "."))
 })
 
-context("Decomposing compositions")
-
 test_that("tree structure of composition preserved when converting to list", {
   f <- identity %>>>%
     mix: (sample %>>>% glue: paste) %>>>%
@@ -472,8 +468,6 @@ test_that("compose() inverts as.list()", {
     expect_equal(cmp1(val), cmp2(val))
 })
 
-context("Generic methods")
-
 sq <- function(x) x^2
 foo <- sq %>>>% inc:{. + 1} %>>>% log:log
 
@@ -918,3 +912,42 @@ test_that("unlist() flattens the list of composite functions", {
     list(abs, log, list(sum = sum, identity))
   )
 })
+
+test_that("composition call is passed on to the initial function", {
+  f <- function(x, y = "y", ..., n) as.list(sys.call(n))[-1]
+
+  # Ensure that the length of the composition is immaterial.
+  for (g in list(NULL, identity, identity, identity)) {
+    f <- compose(f, g)
+    args <- alist(y = 1, 2, n = -length(f), 3)
+    expect_equal(f(y = 1, 2, n = -length(f), 3), args)
+    expect_equal(do.call(f, args), args)
+  }
+})
+
+test_that("argument defaults are evaluated in the conventional environment", {
+  default <- "ignore this"
+  f <- local({
+    default <- "default"
+    function(x = default, y = derived_from(x)) {
+      x <- toupper(x)
+      c(x, y)
+    }
+  })
+  derived_from <- function(x) paste("derived from", x)
+  cmp <- compose(f, identity)
+  expect_equal(cmp(), c("DEFAULT", "derived from DEFAULT"))
+  expect_equal(cmp("given"), c("GIVEN", "derived from GIVEN"))
+  expect_equal(cmp(y = "given"), c("DEFAULT", "given"))
+  expect_equal(cmp("given", "given"), c("GIVEN", "given"))
+})
+
+test_that("missing arguments remain missing under function composition", {
+  is_missing <- compose(function(x = NULL) missing(x), identity)
+  expect_true(is_missing())
+  expect_false(is_missing(NULL))
+
+  is_missing <- compose(function(x) missing(x), identity)
+  expect_true(is_missing())
+  expect_false(is_missing(NULL))
+})
